/*
 * Copyright (c) 2012 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*jslint devel:true, unparam:true*/
/*global Config, Model, Ui, app, $ */

var App = null;

(function strict() { // strict mode wrapper
    'use strict';

    /**
     * Creates a new application object.
     *
     * @class Application
     */
    App = function App() {
        return false;
    };

    App.prototype = {
        /**
         * @type Array
         */
        requires: [
            'js/app.config.js',
            'js/app.model.js',
            'js/app.ui.js',
            'js/app.ui.templateManager.js',
            'js/app.ui.templateManager.modifiers.js'
        ],
        /**
         * @type Config
         */
        config: null,
        /**
         * @type Model
         */
        model: null,
        /**
         * @type Ui
         */
        ui: null,
        /**
         * @type bool
         */
        allDay: false,
        /**
         * @type bool
         */
        alarm: false,
        /**
         * @type CalendarAlarm
         */
        alarmN: null,
        /**
         * @type Date
         */
        lastDateLoaded: null,
        /**
         * @type Integer
         */
        eventId: 0,

        homeDateFilter: new Date(),

        /**
         * Initializes the app.
         */
        init: function init() {
            // instantiate the libs
            this.config = new Config();
            this.model = new Model();
            this.ui = new Ui();

            // initialise the modules
            this.model.init(this);
            this.ui.init(this);

            return this;
        },

        /**
         * Application exit from model.
         */
        exit: function exit() {
            this.model.exit();
        },

        /**
         * Toggle this.allDay
         * @return {boolean} variable state after the toggle
         */
        switchAllDay: function switchAllDay() {
            this.allDay = !this.allDay;
            return this.allDay;
        },

        /**
         * Read initial alarm setting.
         */
        setAlarm: function setAlarm(duration) {
            if (duration !== -1) {
                this.alarmN = this.model.getCalendarAlarm(
                    duration,
                    'EventManager Reminder'
                );
                this.alarm = true;
            } else {
                this.alarmN = null;
                this.alarm = false;
            }
        },

        /**
         * Reads the radio buttons and set this.alarm and this.alarmN
         * accordingly.
         */
        switchAlarm: function switchAlarm() {
            var duration = this.ui.alarm.getValue();
            this.setAlarm(duration);
            app.ui.alarm.updateDurationLabel(duration);
            $.mobile.changePage('#new_event');
        },

        /**
         * Calculate time duration in minutes.
         * @param {TimeDuration} duration
         * @return {Number}
         */
        retrieveTimeDurationInMinutes:
            function retrieveTimeDurationInMinutes(duration) {
                switch (duration.unit) {
                case 'MSECS':
                    return (duration.length / (60 * 1000));
                case 'SECS':
                    return (duration.length / 60);
                case 'MINS':
                    return duration.length;
                case 'HOURS':
                    return (duration.length * 60);
                case 'DAYS':
                    return (duration.length * 60 * 24);
                }
                return duration.length;
            },

        /**
         * Creates a new event in the default calendar,
         * based on values found in #title, #des, #location
         * and this.allDay variable.
         * @param {function} callback
         */
        addEvent: function addEvent(callback) {
            var startDateString = '',
                endDateString = '',
                startDate = null,
                isAllDay = false,
                duration = 0,
                calendarItemInit = null;

            startDateString = this.ui.home.getStartDate();
            endDateString = this.ui.home.getEndDate();
            isAllDay = this.ui.home.getAllDayInfo();
            if (isAllDay) {
                startDateString = startDateString
                    .replace(/[0-9]+:[0-9]+:[0-9]+/, '00:00:00');
                endDateString = endDateString
                    .replace(/[0-9]+:[0-9]+:[0-9]+/, '00:00:00');
            }

            // get duration in minutes
            duration = this.calculateDuration(
                startDateString,
                endDateString,
                isAllDay
            );

            startDate = this.createTZDateFromString(startDateString);
            startDate.setSeconds(0);

            calendarItemInit = {
                startDate: startDate,
                isAllDay: isAllDay,
                duration: duration,
                summary: this.ui.home.getTitle(),
                status: 'TENTATIVE',
                priority: 'MEDIUM'
            };
            this.calendarItemInit = calendarItemInit;

            if (this.alarmN) {
                calendarItemInit.alarms = [this.alarmN];
            }
            try {
                this.model.addEventToDefaultCalendar(calendarItemInit);
            } catch (ex) {
                console.error(ex.message);
            }

            this.changeHomeDateFilter(new Date(startDateString));
            this.loadEvents(startDate);
            if (typeof callback === 'function') {
                callback();
            }
        },

        /**
         * Updates event with data taken from the UI.
         * @param {function} callback
         */
        updateEvent: function updateEvent(callback) {
            var new_values, selectedDate, duration, allDay;

            selectedDate = this.ui.home.getStartDate();
            allDay = this.ui.home.getAllDayInfo();

            duration = this.calculateDuration(
                selectedDate,
                this.ui.home.getEndDate(),
                allDay
            );

            new_values = {
                startDate: this.createTZDateFromString(selectedDate),
                duration: duration,
                summary: this.ui.home.getTitle(),
                isAllDay: allDay,
                alarms: []
            };

            if (this.alarmN) {
                new_values.alarms = [this.alarmN];
            }
            this.model.updateEvent(app.eventId, new_values);

            this.changeHomeDateFilter(new Date(selectedDate));
            this.loadEvents();
            if (typeof callback === 'function') {
                callback();
            }
        },

        /**
         * Changes the date filter.
         * @param {Date} date
         */
        changeHomeDateFilter: function changeHomeDateFilter(date) {
            this.homeDateFilter = date;
            app.ui.home.updateHomeDateFilter();
        },

        /**
         * Calculates time duration.
         *
         * @param {string} startDate
         * @param {string} endDate
         * @param {bool=} allDay 'false' by default
         * @return {TimeDuration}
         */
        calculateDuration: function calculateDuration(
            startDate,
            endDate,
            allDay
        ) {
            var minutes;
            startDate = new Date(startDate);
            endDate = new Date(endDate);
            // duration in minutes
            minutes = Math.round(
                (endDate.getTime() - startDate.getTime()) / 60000
            );
            if (allDay) {
                // round up to full days
                minutes = Math.ceil(minutes / (60 * 24) + 1) * (60 * 24);
            }
            return this.model.getTimeDuration(minutes);
        },

        /**
         * Creates a TZDate object for the given date string, all assuming
         * using the local timezone
         *
         * @param {string} dateString Local date/datetime
         */
        createTZDateFromString: function createTZDateFromString(dateString) {
            var date = null,
                tzDate = null;
            date = new Date(dateString);
            tzDate = this.model.createTZDateFromDate(date);
            return tzDate;
        },

        /**
         * Loads all scheduled events to UI.
         */
        loadEvents: function loadEvents() {
            this.model.getEventsFromDefaultCalendar(
                app.homeDateFilter,
                // Load events into the UI
                this.ui.home.onEventSearchSuccess.bind(this.ui.home),
                this.ui.home.onEventSearchError.bind(this.ui.home)
            );
        },

        /**
         * Checks if delete popup is active closes it if necessary.
         */
        checkDeletePopup: function checkDeletePopup() {
            if ($.mobile.popup.active) {
                this.model.eventExists(app.eventId, null, function close() {
                    $('#popup').popup('close');
                    app.eventId = 0;
                });
            }
        }
    };
}());
